@@ -12,7 +12,9 @@ use std::time::Instant;
 use tokio::net::TcpListener;
 use tokio::sync::Mutex;
 #[allow(unused_imports, dead_code)]
-use tracing::{debug, error, info, instrument, span, trace, warn, Level};
+use tracing::{debug, error, info, instrument, span, trace, warn, event, Level};
+use ate_auth::service::AuthService;
+use ate_auth::cmd::gather_command;
 
 use hyper;
 use hyper::header::HeaderValue;
@@ -27,6 +29,7 @@ use hyper_tungstenite::WebSocketStream;
 
 use ate::prelude::*;
 use ate_files::prelude::*;
+use ate_files::repo::*;
 
 use crate::model::WebConf;
 
@@ -37,7 +40,6 @@ use super::conf::*;
 use super::error::WebServerError;
 use super::error::WebServerErrorKind;
 use super::model::*;
-use super::repo::*;
 use super::stream::*;
 
 pub struct ServerWebConf {
@@ -98,14 +100,49 @@ async fn process(
 impl Server {
     pub(crate) async fn new(builder: ServerBuilder) -> Result<Arc<Server>, AteError> {
         let registry = Arc::new(Registry::new(&builder.conf.cfg_ate).await);
+
+        let session_factory = {
+            let auth_url = builder.auth_url.clone();
+            let registry = registry.clone();
+            let master_key = builder.web_master_key.clone();
+            move |sni: &str, _: ChainKey| {
+                // Create the session
+                let key_entropy = format!("{}:{}", "web-read", sni);
+                let key_entropy = AteHash::from_bytes(key_entropy.as_bytes());
+                let mut session = AteSessionUser::default();
+
+                // If we have a master key then add it
+                if let Some(master_key) = &master_key {
+                    let read_key = AuthService::compute_super_key_from_hash(master_key, &key_entropy);
+                    session.add_user_read_key(&read_key);
+                }
+
+                // Now attempt to gather permissions to the chain
+                let sni = sni.to_string();
+                let registry = registry.clone();
+                let auth_url = auth_url.clone();
+                async move {
+                    // Now we gather the rights to the particular domain this website is running under
+                    Ok(AteSessionType::Group(gather_command(
+                        &registry,
+                        sni,
+                        AteSessionInner::User(session),
+                        auth_url.clone(),
+                    )
+                    .await?))
+                }
+            }
+        };
+
         let repo = Repository::new(
             &registry,
             builder.remote.clone(),
             builder.auth_url.clone(),
-            builder.web_key.clone(),
+            session_factory,
             builder.conf.ttl,
         )
         .await?;
+
         Ok(Arc::new(Server {
             repo,
             web_conf: Mutex::new(FxHashMap::default()),
@@ -220,10 +257,11 @@ impl Server {
             }
         };
         if trigger {
+            let key = ChainKey::from(format!("{}/www", host));
             conf.web_conf_when = Some(Instant::now());
             conf.web_conf = match self
                 .repo
-                .get_file(host.as_str(), WEB_CONF_FILES_CONF)
+                .get_file(&key, host.as_str(), WEB_CONF_FILES_CONF)
                 .await
                 .ok()
                 .flatten()
@@ -244,10 +282,10 @@ impl Server {
                     if let Some(ret_str) = serde_yaml::to_string(&ret).ok() {
                         let err = self
                             .repo
-                            .set_file(host.as_str(), WEB_CONF_FILES_CONF, ret_str.as_bytes())
+                            .set_file(&key, host.as_str(), WEB_CONF_FILES_CONF, ret_str.as_bytes())
                             .await;
                         if let Err(err) = err {
-                            warn!("failed to save default web.yaml - {}", err);
+                            info!("failed to save default web.yaml - {}", err);
                         }
                     }
                     ret
@@ -358,7 +396,8 @@ impl Server {
         conf: &WebConf,
     ) -> Result<Option<Response<Body>>, WebServerError> {
         self.sanitize(path)?;
-        if let Some(data) = self.repo.get_file(host, path).await? {
+        let key = ChainKey::from(format!("{}/www", host));
+        if let Some(data) = self.repo.get_file(&key, host, path).await? {
             let len_str = data.len().to_string();
 
             let mut resp = if is_head {
@@ -572,14 +611,20 @@ impl Server {
             return self.process_upgrade(req, sock_addr).await;
         }
 
-        let is_head = req.method() == Method::HEAD;
+        let uri = req.uri().clone();
+        let method = req.method().clone();
+        let is_head = method == Method::HEAD;
         let host = self.get_host(&req)?;
         let conf = self.get_conf(host.as_str()).await?;
 
         let ret = self.process_internal(req, listen, &conf).await;
         match ret {
-            Ok(a) => Ok(a),
+            Ok(a) => {
+                info!("http peer={} method={} path={} - {}", sock_addr, method, uri, a.status());
+                Ok(a)
+            },
             Err(err) => {
+                info!("http peer={} method={} path={} err={}", sock_addr, method, uri, err);
                 let page = conf
                     .status_pages
                     .get(&err.status_code().as_u16())
@@ -610,11 +655,11 @@ impl Server {
                     Ok(websocket) => {
                         let ret = callback.web_socket(websocket, sock_addr).await;
                         if let Err(err) = ret {
-                            error!("web socket failed - {}", err);
+                            error!("web socket failed(1) - {}", err);
                         }
                     }
                     Err(err) => {
-                        error!("web socket failed - {}", err);
+                        error!("web socket failed(2) - {}", err);
                     }
                 }
             });
